@@ -3,6 +3,7 @@
 
 #include "certs.h"
 #include "mongoose.h"
+#include <zephyr/net/net_mgmt.h>
 
 static const char *s_web_dir = "/";
 static const char *s_ws_addr = "ws://0.0.0.0:8000";
@@ -17,7 +18,7 @@ static void fn(struct mg_connection *c, int ev, void *ev_data) {
   if (ev == MG_EV_OPEN) {
     c->is_hexdumping = 1;
   } else if (ev == MG_EV_ACCEPT && c->fn_data != NULL) {
-    struct mg_tls_opts opts = {.cert = s_ssl_cert, .key = s_ssl_key};
+    struct mg_tls_opts opts = {.cert = (char *) s_ssl_cert, .key = (char *) s_ssl_key};
     mg_tls_init(c, &opts);
   } else if (ev == MG_EV_HTTP_MSG) {
     struct mg_http_message *hm = (struct mg_http_message *) ev_data;
